<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_Faq
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.Pinpoint.co.uk)
 * @author      Ahmed Goda <hello@Pinpoint.co.uk>
 */

namespace Pinpoint\Faq\Controller\Adminhtml\Faq;

use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\View\Result\Page;
use Magento\Backend\Model\View\Result\Redirect;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\Registry;
use Magento\Framework\View\Result\PageFactory;
use Pinpoint\Faq\Api\Data\FaqInterface;
use Pinpoint\Faq\Api\FaqRepositoryInterface;
use Pinpoint\Faq\Controller\Adminhtml\Faq;

/**
 * Class Edit
 */
class Edit extends Faq implements HttpGetActionInterface
{
    /**
     * @var PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var FaqRepositoryInterface
     */
    private $faqRepository;

    /**
     * @param Context $context
     * @param Registry $coreRegistry
     * @param PageFactory $resultPageFactory
     * @param FaqRepositoryInterface $faqRepository
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        PageFactory $resultPageFactory,
        FaqRepositoryInterface $faqRepository
    ) {
        parent::__construct($context, $coreRegistry);
        $this->resultPageFactory = $resultPageFactory;
        $this->faqRepository = $faqRepository;
    }

    /**
     * @return Page|Redirect|\Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $id = $this->getRequest()->getParam(FaqInterface::FAQ_ID);
        $model = null;

        if ($id) {
            $model = $this->faqRepository->getById($id);
            if (!$model) {
                $this->messageManager->addErrorMessage(
                    __('This question no longer exists.')
                );
                return $this->resultRedirectFactory->create()->setPath('*/*/');
            }
        }
        $this->coreRegistry->register('pinpoint_faqs', $model);

        /** @var Page $resultPage */
        $resultPage = $this->resultPageFactory->create();

        $this->initPage($resultPage)->addBreadcrumb(
            $id ? __('Edit Question') : __('New Question'),
            $id ? __('Edit Question') : __('New Question')
        );

        $this->setPageTitle($resultPage, $model);

        return $resultPage;
    }

    /**
     * @param Page $resultPage
     * @param FaqInterface|null $model
     */
    protected function setPageTitle(Page $resultPage, ?FaqInterface $model): void
    {
        $resultPage->getConfig()->getTitle()->prepend(__('FAQs'));

        if ($model instanceof FaqInterface) {
            $resultPage->getConfig()->getTitle()->prepend($model->getQuestionText());
        } else {
            $resultPage->getConfig()->getTitle()->prepend(__('Add Question'));
        }
    }
}
